@@ -18,7 +18,6 @@
 import datetime
 
 from py4j.compat import long
-from typing import Tuple
 
 from pyflink.common.configuration import Configuration
 from pyflink.java_gateway import get_gateway
@@ -26,18 +25,48 @@ from pyflink.table.sql_dialect import SqlDialect
 
 __all__ = ['TableConfig']
 
+from pyflink.util.java_utils import add_jars_to_context_class_loader
+
 
 class TableConfig(object):
     """
     Configuration for the current :class:`TableEnvironment` session to adjust Table & SQL API
     programs.
 
+    This class is a pure API class that abstracts configuration from various sources. Currently,
+    configuration can be set in any of the following layers (in the given order):
+
+    - flink-conf.yaml
+    - CLI parameters
+    - :class:`~pyflink.datastream.StreamExecutionEnvironment` when bridging to DataStream API
+    - :func:`~EnvironmentSettings.Builder.with_configuration`
+    - :func:`~TableConfig.set`
+
+    The latter two represent the application-specific part of the configuration. They initialize
+    and directly modify :func:`~TableConfig.get_configuration`. Other layers represent the
+    configuration of the execution context and are immutable.
+
+    The getter :func:`~TableConfig.get` gives read-only access to the full configuration. However,
+    application-specific configuration has precedence. Configuration of outer layers is used for
+    defaults and fallbacks. The setter :func:`~TableConfig.set` will only affect
+    application-specific configuration.
+
     For common or important configuration options, this class provides getters and setters methods
     with detailed inline documentation.
 
     For more advanced configuration, users can directly access the underlying key-value map via
     :func:`~pyflink.table.TableConfig.get_configuration`.
 
+    Example:
+    ::
+
+        >>> table_config = t_env.get_config()
+        >>> config = Configuration()
+        >>> config.set_string("parallelism.default", "128") \\
+        ...       .set_string("pipeline.auto-watermark-interval", "800ms") \\
+        ...       .set_string("execution.checkpointing.interval", "30s")
+        >>> table_config.add_configuration(config)
+
     .. note::
 
         Because options are read at different point in time when performing operations, it is
@@ -47,10 +76,40 @@ class TableConfig(object):
     def __init__(self, j_table_config=None):
         gateway = get_gateway()
         if j_table_config is None:
-            self._j_table_config = gateway.jvm.TableConfig()
+            self._j_table_config = gateway.jvm.TableConfig.getDefault()
         else:
             self._j_table_config = j_table_config
 
+    def get(self, key: str, default_value: str) -> str:
+        """
+        Returns the value associated with the given key as a string.
+
+        :param key: The key pointing to the associated value.
+        :param default_value: The default value which is returned in case there is no value
+                              associated with the given key.
+        :return: The (default) value associated with the given key.
+        """
+        if self.get_configuration().contains_key(key):
+            return self.get_configuration().get_string(key, default_value)
+        else:
+            return self._j_table_config.getRootConfiguration().getString(key, default_value)
+
+    def set(self, key: str, value: str) -> 'TableConfig':
+        """
+        Sets a string-based value for the given string-based key.
+
+        The value will be parsed by the framework on access.
+        """
+        self._j_table_config.set(key, value)
+
+        jvm = get_gateway().jvm
+        jars_key = jvm.org.apache.flink.configuration.PipelineOptions.JARS.key()
+        classpaths_key = jvm.org.apache.flink.configuration.PipelineOptions.CLASSPATHS.key()
+        if key in [jars_key, classpaths_key]:
+            add_jars_to_context_class_loader(value.split(";"))
+
+        return self
+
     def get_local_timezone(self) -> str:
         """
         Returns the local timezone id for timestamp with local time zone, either an abbreviation
@@ -73,21 +132,6 @@ class TableConfig(object):
         else:
             raise Exception("TableConfig.timezone should be a string!")
 
-    def get_null_check(self) -> bool:
-        """
-        A boolean value, "True" enables NULL check and "False" disables NULL check.
-        """
-        return self._j_table_config.getNullCheck()
-
-    def set_null_check(self, null_check: bool):
-        """
-        Sets the NULL check. If enabled, all fields need to be checked for NULL first.
-        """
-        if null_check is not None and isinstance(null_check, bool):
-            self._j_table_config.setNullCheck(null_check)
-        else:
-            raise Exception("TableConfig.null_check should be a bool value!")
-
     def get_max_generated_code_length(self) -> int:
         """
         The current threshold where generated code will be split into sub-function calls. Java has
@@ -167,8 +211,7 @@ class TableConfig(object):
         Example:
         ::
 
-            >>> table_config = TableConfig() \\
-            ...     .set_idle_state_retention(datetime.timedelta(days=1))
+            >>> table_config.set_idle_state_retention(datetime.timedelta(days=1))
 
         .. note::
 
@@ -221,79 +264,6 @@ class TableConfig(object):
         return datetime.timedelta(
             milliseconds=self._j_table_config.getIdleStateRetention().toMillis())
 
-    def set_decimal_context(self, precision: int, rounding_mode: str):
-        """
-        Sets the default context for decimal division calculation.
-        (precision=34, rounding_mode=HALF_EVEN) by default.
-
-        The precision is the number of digits to be used for an operation. A value of 0 indicates
-        that unlimited precision (as many digits as are required) will be used. Note that leading
-        zeros (in the coefficient of a number) are never significant.
-
-        The rounding mode is the rounding algorithm to be used for an operation. It could be:
-
-        **UP**, **DOWN**, **CEILING**, **FLOOR**, **HALF_UP**, **HALF_DOWN**, **HALF_EVEN**,
-        **UNNECESSARY**
-
-        The table below shows the results of rounding input to one digit with the given rounding
-        mode:
-
-        +-------+----+------+---------+-------+---------+-----------+-----------+-------------+
-        | Input | UP | DOWN | CEILING | FLOOR | HALF_UP | HALF_DOWN | HALF_EVEN | UNNECESSARY |
-        +=======+====+======+=========+=======+=========+===========+===========+=============+
-        | 5.5   |  6 |   5  |    6    |   5   |    6    |     5     |     6     |  Exception  |
-        +-------+----+------+---------+-------+---------+-----------+-----------+-------------+
-        | 2.5   |  3 |   2  |    3    |   2   |    3    |     2     |     2     |  Exception  |
-        +-------+----+------+---------+-------+---------+-----------+-----------+-------------+
-        | 1.6   |  2 |   1  |    2    |   1   |    2    |     2     |     2     |  Exception  |
-        +-------+----+------+---------+-------+---------+-----------+-----------+-------------+
-        | 1.1   |  2 |   1  |    2    |   1   |    1    |     1     |     1     |  Exception  |
-        +-------+----+------+---------+-------+---------+-----------+-----------+-------------+
-        | 1.0   |  1 |   1  |    1    |   1   |    1    |     1     |     1     |      1      |
-        +-------+----+------+---------+-------+---------+-----------+-----------+-------------+
-        | -1.0  | -1 |  -1  |   -1    |  -1   |   -1    |    -1     |    -1     |     -1      |
-        +-------+----+------+---------+-------+---------+-----------+-----------+-------------+
-        | -1.1  | -2 |  -1  |   -1    |  -2   |   -1    |    -1     |    -1     |  Exception  |
-        +-------+----+------+---------+-------+---------+-----------+-----------+-------------+
-        | -1.6  | -2 |  -1  |   -1    |  -2   |   -2    |    -2     |    -2     |  Exception  |
-        +-------+----+------+---------+-------+---------+-----------+-----------+-------------+
-        | 2.5   | -3 |  -2  |   -2    |  -3   |   -3    |    -2     |    -2     |  Exception  |
-        +-------+----+------+---------+-------+---------+-----------+-----------+-------------+
-        | 5.5   | -6 |  -5  |   -5    |  -6   |   -6    |    -5     |    -6     |  Exception  |
-        +-------+----+------+---------+-------+---------+-----------+-----------+-------------+
-
-        :param precision: The precision of the decimal context.
-        :param rounding_mode: The rounding mode of the decimal context.
-        """
-        if rounding_mode not in (
-                "UP",
-                "DOWN",
-                "CEILING",
-                "FLOOR",
-                "HALF_UP",
-                "HALF_DOWN",
-                "HALF_EVEN",
-                "UNNECESSARY"):
-            raise ValueError("Unsupported rounding_mode: %s" % rounding_mode)
-        gateway = get_gateway()
-        j_rounding_mode = getattr(gateway.jvm.java.math.RoundingMode, rounding_mode)
-        j_math_context = gateway.jvm.java.math.MathContext(precision, j_rounding_mode)
-        self._j_table_config.setDecimalContext(j_math_context)
-
-    def get_decimal_context(self) -> Tuple[int, str]:
-        """
-        Returns current context for decimal division calculation,
-        (precision=34, rounding_mode=HALF_EVEN) by default.
-
-        .. seealso:: :func:`set_decimal_context`
-
-        :return: the current context for decimal division calculation.
-        """
-        j_math_context = self._j_table_config.getDecimalContext()
-        precision = j_math_context.getPrecision()
-        rounding_mode = j_math_context.getRoundingMode().name()
-        return precision, rounding_mode
-
     def get_configuration(self) -> Configuration:
         """
         Gives direct access to the underlying key-value map for advanced configuration.
@@ -355,7 +325,7 @@ class TableConfig(object):
 
         .. note::
 
-            The python udf worker depends on Apache Beam (version == 2.27.0).
+            The python udf worker depends on Apache Beam (version == 2.38.0).
             Please ensure that the specified environment meets the above requirements.
 
         :param python_exec: The path of python interpreter.
@@ -363,7 +333,7 @@ class TableConfig(object):
         .. versionadded:: 1.10.0
         """
         jvm = get_gateway().jvm
-        self.get_configuration().set_string(jvm.PythonOptions.PYTHON_EXECUTABLE.key(), python_exec)
+        self.set(jvm.PythonOptions.PYTHON_EXECUTABLE.key(), python_exec)
 
     def get_python_executable(self) -> str:
         """
