@@ -24,7 +24,7 @@ from typing import Union, List, Type, Callable, TypeVar, Generic, Iterable
 from pyflink.java_gateway import get_gateway
 from pyflink.metrics import MetricGroup
 from pyflink.table import Expression
-from pyflink.table.types import DataType, _to_java_type, _to_java_data_type
+from pyflink.table.types import DataType, _to_java_data_type
 from pyflink.util import java_utils
 
 __all__ = ['FunctionContext', 'AggregateFunction', 'ScalarFunction', 'TableFunction',
@@ -378,7 +378,7 @@ class UserDefinedFunctionWrapper(object):
 
             if self._input_types is not None:
                 j_input_types = java_utils.to_jarray(
-                    gateway.jvm.TypeInformation, [_to_java_type(i) for i in self._input_types])
+                    gateway.jvm.DataType, [_to_java_data_type(i) for i in self._input_types])
             else:
                 j_input_types = None
             j_function_kind = get_python_function_kind()
@@ -416,7 +416,7 @@ class UserDefinedScalarFunctionWrapper(UserDefinedFunctionWrapper):
 
     def _create_judf(self, serialized_func, j_input_types, j_function_kind):
         gateway = get_gateway()
-        j_result_type = _to_java_type(self._result_type)
+        j_result_type = _to_java_data_type(self._result_type)
         PythonScalarFunction = gateway.jvm \
             .org.apache.flink.table.functions.python.PythonScalarFunction
         j_scalar_function = PythonScalarFunction(
@@ -458,9 +458,9 @@ class UserDefinedTableFunctionWrapper(UserDefinedFunctionWrapper):
 
     def _create_judf(self, serialized_func, j_input_types, j_function_kind):
         gateway = get_gateway()
-        j_result_types = java_utils.to_jarray(gateway.jvm.TypeInformation,
-                                              [_to_java_type(i) for i in self._result_types])
-        j_result_type = gateway.jvm.org.apache.flink.api.java.typeutils.RowTypeInfo(j_result_types)
+        j_result_types = java_utils.to_jarray(gateway.jvm.DataType,
+                                              [_to_java_data_type(i) for i in self._result_types])
+        j_result_type = gateway.jvm.DataTypes.ROW(j_result_types)
         PythonTableFunction = gateway.jvm \
             .org.apache.flink.table.functions.python.PythonTableFunction
         j_table_function = PythonTableFunction(
